package Uploader::GUI::DlgCommon;
use strict;
use Wx::Calendar;

use base qw/
               Class::Accessor::Fast
           /; 

__PACKAGE__->mk_accessors(     
    qw/
        properties
      /
);

sub InitHandler {
    my ( $self ) = @_;

	# to connect the right event handler to each control
	my $ctrl_handlers = {
	    'Wx::TextCtrl' => sub { my ( $ctrl ) = @_; Wx::Event::EVT_TEXT( $ctrl, $ctrl, sub { $self->OnTextCtrl(@_) } ); },
	    'Wx::Choice' => sub { my ( $ctrl ) = @_; Wx::Event::EVT_CHOICE( $ctrl, $ctrl, sub { $self->OnChoice(@_) } ); },
	    'Wx::DatePickerCtrl' => sub { my ( $ctrl ) = @_; Wx::Event::EVT_DATE_CHANGED( $ctrl, $ctrl, sub { $self->OnDatePicker(@_) } ); },
	};
	
	map {
        my $ctrl =$self->FindWindow($_);
        if(defined $ctrl){
		    $ctrl_handlers->{ ref $ctrl}->($ctrl) if exists $ctrl_handlers->{ ref $ctrl};
		}
    }
	keys %{$self->properties};
	

}

# what does happen when the text changes
sub OnTextCtrl {
    my ( $self, $ctrl, $event ) = @_;
	
    my $id = $ctrl->GetId;
    # change the property value
	$self->properties->{$id}->{value}->(
        $event->GetString
    ) if exists $self->properties->{$id}->{value};

    # exec the callback
    $self->properties->{$id}->{frame_callback}->() if exists
        $self->properties->{$id}->{frame_callback};	
	
}

sub OnChoice {
    my ( $self, $ctrl, $event ) = @_;
	
    my $id = $ctrl->GetId;
    # change the property value
	$self->properties->{$id}->{selection}->(
        $event->GetSelection
    ) if exists $self->properties->{$id}->{selection};

    # exec the callback
    $self->properties->{$id}->{frame_callback}->() if exists
        $self->properties->{$id}->{frame_callback};	
	
}

sub OnDatePicker {
    my ( $self, $ctrl, $event ) = @_;
	
    my $id = $ctrl->GetId;

    # change the property value
	$self->properties->{$id}->{value}->(
        $event->GetDate->FormatISODate
    ) if exists $self->properties->{$id}->{value};
    # exec the callback
    $self->properties->{$id}->{frame_callback}->() if exists
        $self->properties->{$id}->{frame_callback};	
	
}


my $change_value = {
    'Wx::TextCtrl' => sub { my ($ctrl, $value ) = @_; $ctrl->ChangeValue($value)},
    'Wx::CheckBox' => sub { my ($ctrl, $value ) = @_; $ctrl->SetValue($value)},
    'Wx::DatePickerCtrl' => sub { my ($ctrl, $value ) = @_;
                                  my $date = Wx::DateTime->new;	
                                  my ($yyyy, $mm, $dd, $hh, $mi, $ss ) = split(/[:\/\\\-\.\s]/, $value);
								  $date->ParseDate(
								      sprintf("%s/%s/%s", $yyyy, $mm, $dd)
								  );
	                              $ctrl->SetValue($date);
	                            },
};	

sub SetProperties {
    my ( $self ) = @_;
	
	map {
        my $ctrl =$self->FindWindow($_);
        if(defined $ctrl){
 	        # checkbox, static text
            $change_value->{ref $ctrl}->(
		        $ctrl,
                $self->properties->{$_}->{value}->()
		    ) if exists $self->properties->{$_}->{value};
		    # only works for control with items
		    $ctrl->SetSelection(
		        $self->properties->{$_}->{selection}->()			
		    ) if exists $self->properties->{$_}->{selection};
		}
    }
	keys %{$self->properties};
}

sub InitLabels {
    my ( $self ) = @_;
	
	map {
        my $ctrl =$self->FindWindow($_);
        #printf("ctrl %s : %s\n", $_, $ctrl);		
        if(defined $ctrl){
  	        # checkbox, static text
		    $ctrl->SetLabel(
               $self->properties->{$_}->{label}
		    ) if exists $self->properties->{$_}->{label};
		
            # radiobox
            my $labels =$self->properties->{$_}->{labels};
		    $labels||=[];
		    for(my $i=0; $i < scalar @$labels ; $i++){
		        $ctrl->SetItemLabel($i, $labels->[$i]);
		    }
		}
    }
	keys %{$self->properties};

}

sub InitChoices {
    my ( $self ) = @_;

		map {
        my $ctrl =$self->FindWindow($_);
        #printf("ctrl %s : %s\n", $_, $ctrl);		
        if(defined $ctrl){
            # choice
            my $choices =$self->properties->{$_}->{choices};
		    $choices||=[];
		    map{
		        $ctrl->Append($_);
		    }@$choices;
		}
    }
	keys %{$self->properties};

}

sub GetProperties {
    my ( $self ) = @_;

	map {
        my $ctrl = $self->FindWindow($_);
        #printf("ctrl %s : %s\n", $_, $ctrl);		
        if(defined $ctrl){
		# checkbox, static text
            $self->properties->{$_}->{value}->(
                $ctrl->GetValue()
		    ) if exists $self->properties->{$_}->{value};

            $self->properties->{$_}->{selection}->(
		        $ctrl->GetSelection()			
		    ) if exists $self->properties->{$_}->{selection};
		}
    }
	keys %{$self->properties};
}

1;